/*********************************************************************
 * Job Ticket API (JTAPI) fsgjt_job_ticket_info.h                                           
 *                                                                   
 * Copyright (c) 2004 High North Inc.                                                                  
 * Copyright (c) 2004 International Business Machines Corp.
 * Copyright (c) 2004 Seiko-Epson, Inc.          
 * Copyright (c) 2004 Till Kamppeter.  
 * Copyright (c) 2004 Free Standards Group.          
 *                                                                    
 *                    Common Public License - v 1.0 
 * THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS COMMON 
 * PUBLIC LICENSE ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION 
 * OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.   
 *
 * The Common Public License in its entirety is located in 
 * fsgjt_cpl.h and applies to the contents of this file. 
 *********************************************************************/

#ifndef FSGJT_JOB_TICKET_INFO_H
#define FSGJT_JOB_TICKET_INFO_H

#include <stdio.h>
#include <fsgjt.h>

/* Defined constant to uniquely identify this type of JTAPI object */
#define FSGJT_JOB_TICKET_INFO_OBJECT    "JTIC"

/************************************************************
 * Job Ticket Info attributes
 ***********************************************************/
/* Single value where type is char* */
#define FSGJT_JT_API_VERSION      "jt-api-version"

/* Single value where type is char* */
#define FSGJT_JT_COMMENT          "jt-comment"

/* Single value where type is fsgjt_object_t (Job) */
#define FSGJT_JT_JOB              "jt-job"

/* Single value where type is fsgjt_length_unit_t 
   (LengthUnitEnum) */
#define FSGJT_JT_LENGTH_UNIT      "jt-length-unit"

/* Single value where type is char* (JobTicketTypeEnum) */
#define FSGJT_JT_TYPE_AND_VERSION "jt-type-and-version"

/************************************************************
 * fsgjtNewJobTicketInfo - Creates a JobTicketInfo object.
 * Used when a new job ticket is being created without
 * a Job object and not from an existing job ticket.
 *
 * param (output) fsgjt_object_t* jobTicketInfo - Pointer to 
 *      the new JobTicketInfo to create
 * param (input) char* typeAndVersion - Type and version of 
 *      this job ticket (use JobTicketTypeVersionEnum).
 *
 * return fsgjt_return_code_t
 ***********************************************************/
fsgjt_return_code_t fsgjtNewJobTicketInfo(
               fsgjt_object_t* jobTicketInfo,
               char* typeAndVersion);

/************************************************************
 * fsgjtNewJobTicketInfoFromURI - Creates a JobTicketInfo 
 * object from the job ticket file at the provided URI. Other 
 * JTAPI objects are also created based on the information in
 * the job ticket file.
 *
 * param (output) fsgjt_object_t* jobTicketInfo - Pointer to 
 *      the JobTicketInfo to create
 * param (input) fsgjt_uri_t uri - URI where job ticket file 
 *      resides
 * param (input) char* typeAndVersion - Type and version of 
 *      this job ticket (use JobTicketTypeVersionEnum).
 *
 * return fsgjt_status_t
 ***********************************************************/
fsgjt_status_t fsgjtNewJobTicketInfoFromURI(
               fsgjt_object_t* jobTicketInfo,
               fsgjt_uri_t uri,
               char* typeAndVersion);

/************************************************************
 * fsgjtNewJobTicketInfoFromBuffer - Creates a JobTicketInfo 
 * object from the provided buffer that contains a job 
 * ticket. Other JTAPI objects are also created based on the 
 * information in the job ticket buffer.
 *
 * param (output) fsgjt_object_t* jobTicketInfo - Pointer to 
 *      the JobTicketInfo to create
 * param (input) char** buffer - Buffer that contains the 
 *      job ticket
 * param (input) size_t bufferSize - Size of the buffer 
 * param (input) char* typeAndVersion - Type and version of 
 *      this job ticket (use JobTicketTypeVersionEnum).
 *
 * return fsgjt_status_t
 ***********************************************************/
fsgjt_status_t fsgjtNewJobTicketInfoFromBuffer(
               fsgjt_object_t* jobTicketInfo,
               char** buffer,
               size_t bufferSize,
               char* typeAndVersion);

/************************************************************
 * fsgjtNewJobTicketInfoFromJob - Creates a JobTicketInfo 
 * object using the provided Job object. Used when a new job 
 * ticket is being created. The Job object must be created 
 * first.
 *
 * param (output) fsgjt_object_t* jobTicketInfo - Pointer to 
 *      the new JobTicketInfo to create
 * param (input) fsgjt_object_t job - Job object that this 
 *      JobTicketInfo is to contain
 * param (input) char* typeAndVersion - Type and version of 
 *      this job ticket (use JobTicketTypeVersionEnum)
 *
 * return fsgjt_return_code_t
 ***********************************************************/
fsgjt_return_code_t fsgjtNewJobTicketInfoFromJob(
               fsgjt_object_t* jobTicketInfo,
               fsgjt_object_t job,
               char* typeAndVersion);

/************************************************************
 * fsgjtWriteJobTicketToBuffer - Writes a job ticket to the 
 * provided buffer.
 *
 * param (input) fsgjt_object_t jobTicketInfo - JobTicketInfo 
 *      from which the job ticket is to be written
 * param (output) char** buffer - Pointer to buffer to write 
 *      the job ticket to. The buffer must be created by the 
 *      caller.
 * param (input) size_t bufferSize - Size of the buffer that 
 *      caller created
 *
 * return fsgjt_return_code_t
 ***********************************************************/
fsgjt_return_code_t fsgjtWriteJobTicketToBuffer(
               fsgjt_object_t jobTicketInfo,
               char** buffer,
               size_t bufferSize);

/************************************************************
 * fsgjtWriteJobTicketToURI - Wrtes a job ticket at the 
 * provided URI.
 *
 * param (input) fsgjt_object_t jobTicketInfo - JobTicketInfo  
 *      from which the job ticket is to be written
 * param (output) fsgjt_uri_t uri - URI of the job ticket 
 *      that is to be created from the JobTicketInfo. 
 *
 * return fsgjt_return_code_t
 ***********************************************************/
fsgjt_return_code_t fsgjtWriteJobTicketToURI(
               fsgjt_object_t jobTicketInfo,
               fsgjt_uri_t uri);

#endif /* FSGJT_JOB_TICKET_INFO_H */
